
use pubs
go

-- OVER clause

-- Can't mix aggregates and columns without subquery
SELECT a.state
     , avg(royaltyper) as [Average Royalties]
FROM   authors a
JOIN titleauthor ta
  ON a.au_id = ta.au_id

SELECT a.state
     , (SELECT avg(royaltyper) FROM titleauthor) as [Average Total Royalties]
FROM   authors a
JOIN titleauthor ta
  ON a.au_id = ta.au_id

-- Group by can't select individual rows within a group
SELECT a.state
     , avg(royaltyper) as [Average Royalties]
FROM   authors a
JOIN titleauthor ta
  ON a.au_id = ta.au_id
GROUP BY a.state

-- Or include other columns (error)
SELECT a.state
     , a.au_lname
	 , ta.title_id
     , avg(royaltyper) as [Average Royalties]
FROM   authors a
JOIN titleauthor ta
  ON a.au_id = ta.au_id
GROUP BY a.state

-- Need over() clause
SELECT a.state
     , a.au_lname
	 , ta.title_id
	 , ta.royaltyper
     , avg(royaltyper) over() as [Average Total Royalties]
FROM   authors a
JOIN titleauthor ta
  ON a.au_id = ta.au_id

-- and what if you need many aggregates?
SELECT a.state
     , a.au_lname
	 , ta.title_id
	 , ta.royaltyper
     , (SELECT max(royaltyper) FROM titleauthor) - ta.royaltyper as [Diff To Maximum]
     , ta.royaltyper - (SELECT min(royaltyper) FROM titleauthor) as [Diff To Minimum]
	 , ta.royaltyper - (SELECT avg(royaltyper) FROM titleauthor) as [Diff To Average]
FROM   authors a
JOIN titleauthor ta
  ON a.au_id = ta.au_id

-- vs OVER clause
SELECT a.state
     , a.au_lname
	 , ta.title_id
	 , ta.royaltyper
     , max(royaltyper) over() - ta.royaltyper as [Diff To Maximum]
     , ta.royaltyper - min(royaltyper) over() as [Diff To Minimum]
	 , ta.royaltyper - avg(royaltyper) over() as [Diff To Average]
FROM   authors a
JOIN titleauthor ta
  ON a.au_id = ta.au_id

-- or groups without using group by
SELECT a.state
     , avg(royaltyper) over(PARTITION BY STATE) as [Average Royalties By State]
FROM   authors a
JOIN titleauthor ta
  ON a.au_id = ta.au_id

-- all in the same query
SELECT a.state
     , a.au_lname
	 , ta.title_id
	 , ta.royaltyper
     , avg(royaltyper) over() as [Average Total Royalties]
	 , ta.royaltyper - avg(royaltyper) over() as [Percent Diff]
	 , avg(royaltyper) over(PARTITION BY STATE) as [Average Royalties By State]
FROM   authors a
JOIN titleauthor ta
  ON a.au_id = ta.au_id
go

-- SQL 2005 ranking functions
use northwind
go

SELECT orderid, customerid, 
  ROW_NUMBER() OVER(ORDER BY orderid) AS num 
FROM orders
WHERE orderid < 10400
AND   customerid <= 'BN'
GO

-- ties handled differently
SELECT orderid, customerid, 
  ROW_NUMBER() OVER(ORDER BY customerid) AS num,
  RANK()       OVER(ORDER BY customerid) AS [rank],
  DENSE_RANK() OVER(ORDER BY customerid) AS [densr],
  NTILE(5)     OVER(ORDER BY customerid) AS ntile5
FROM  orders
WHERE orderid < 10400
AND   customerid <= 'BN'
GO

-- tiebreaker, no ties
SELECT orderid, customerid, 
  ROW_NUMBER() OVER(ORDER BY customerid, orderid) AS num,
  RANK()       OVER(ORDER BY customerid, orderid) AS [rank],
  DENSE_RANK() OVER(ORDER BY customerid, orderid) AS [densr],
  NTILE(5)     OVER(ORDER BY customerid, orderid) AS ntile5
FROM  orders
WHERE orderid < 10400
AND   customerid <= 'BN'
GO

-- there is one oldest employee age for each country
select *,
  RANK() OVER(PARTITION BY COUNTRY ORDER BY age) as [rank],
  MAX(age) OVER(PARTITION BY COUNTRY) as [oldest age in country]
from
(
 select lastname, country, 
   datediff(yy,birthdate,getdate()) as age
 from employees
) as a	
go
