
use pubs
go

-- OVER clause

-- Can't mix aggregates and columns without subquery
SELECT a.state
     , avg(royaltyper) as [Average Royalties]
FROM   authors a
JOIN titleauthor ta
  ON a.au_id = ta.au_id

SELECT a.state
     , (SELECT avg(royaltyper) FROM titleauthor) as [Average Total Royalties]
FROM   authors a
JOIN titleauthor ta
  ON a.au_id = ta.au_id

-- Group by can't select individual rows within a group
SELECT a.state
     , avg(royaltyper) as [Average Royalties]
FROM   authors a
JOIN titleauthor ta
  ON a.au_id = ta.au_id
GROUP BY a.state

-- Or include other columns (error)
SELECT a.state
     , a.au_lname
	 , ta.title_id
     , avg(royaltyper) as [Average Royalties]
FROM   authors a
JOIN titleauthor ta
  ON a.au_id = ta.au_id
GROUP BY a.state

-- Need over() clause
SELECT a.state
     , a.au_lname
	 , ta.title_id
	 , ta.royaltyper
     , avg(royaltyper) over() as [Average Total Royalties]
FROM   authors a
JOIN titleauthor ta
  ON a.au_id = ta.au_id

-- and what if you need many aggregates?
SELECT a.state
     , a.au_lname
	 , ta.title_id
	 , ta.royaltyper
     , max(royaltyper) over() - ta.royaltyper as [Diff To Minimum]
     , ta.royaltyper - min(royaltyper) over() as [Diff To Minimum]
	 , ta.royaltyper - avg(royaltyper) over() as [Diff To Average]
FROM   authors a
JOIN titleauthor ta
  ON a.au_id = ta.au_id

-- or groups without using group by
SELECT a.state
     , avg(royaltyper) over(PARTITION BY STATE) as [Average Royalties By State]
FROM   authors a
JOIN titleauthor ta
  ON a.au_id = ta.au_id

-- all in the same query
SELECT a.state
     , a.au_lname
	 , ta.title_id
	 , ta.royaltyper
     , avg(royaltyper) over() as [Average Total Royalties]
	 , ta.royaltyper - avg(royaltyper) over() as [Percent Diff]
	 , avg(royaltyper) over(PARTITION BY STATE) as [Average Royalties By State]
FROM   authors a
JOIN titleauthor ta
  ON a.au_id = ta.au_id


/*
use AdventureWorksDW2008R2
go

SELECT * FROM dbo.EmployeeSalesByMonth
ORDER BY Employee, Year, Month
GO

SET STATISTICS IO ON
GO

-- Rolling totals - rolling sales, per employee

-- Pre-SQL Server 2012 with self-join
SELECT S1.Employee, S1.Year, S1.Month, S1.EmployeeTotal AS SalesThisMonth,
  SUM(S2.EmployeeTotal) AS CummulativeSales
FROM dbo.EmployeeSalesByMonth AS S1
  JOIN dbo.EmployeeSalesByMonth AS S2
    ON S2.Employee = S1.Employee
   AND DATEFROMPARTS(S2.Year, S2.Month, 1) <= DATEFROMPARTS(S1.Year, S1.Month, 1)
GROUP BY S1.Employee, S1.Year, S1.Month, S1.EmployeeTotal
ORDER BY S1.Employee, S1.Year, S1.Month;

-- SQL Server 2012 with Window
SELECT Employee, Year, Month, EmployeeTotal AS SalesThisMonth,
  SUM(EmployeeTotal) OVER(PARTITION BY Employee
                          ORDER BY Year, Month
                          ROWS BETWEEN UNBOUNDED PRECEDING AND CURRENT ROW) AS CummulativeSales
FROM dbo.EmployeeSalesByMonth;
*/