
-- This makes the densified table for AdventureWorks2008R2 (not DW as previous example)
-- and saves it as 

use AdventureWorks2008R2
GO

SET NOCOUNT ON;
USE tempdb;

-- GetNums function
IF OBJECT_ID('dbo.GetNums') IS NOT NULL DROP FUNCTION dbo.GetNums;
GO
CREATE FUNCTION dbo.GetNums(@n AS BIGINT) RETURNS TABLE
AS
RETURN
  WITH
    L0   AS(SELECT 1 AS c UNION ALL SELECT 1),
    L1   AS(SELECT 1 AS c FROM L0 AS A, L0 AS B),
    L2   AS(SELECT 1 AS c FROM L1 AS A, L1 AS B),
    L3   AS(SELECT 1 AS c FROM L2 AS A, L2 AS B),
    L4   AS(SELECT 1 AS c FROM L3 AS A, L3 AS B),
    L5   AS(SELECT 1 AS c FROM L4 AS A, L4 AS B),
    Nums AS(SELECT ROW_NUMBER() OVER(ORDER BY (SELECT NULL)) AS n FROM L5)
  SELECT TOP(@n) n FROM Nums ORDER BY n;
GO

SET NOCOUNT OFF;

DECLARE 
 @startdt AS DATE = '20090101',
 @enddt   AS DATE = '20091231'
 
SELECT DATEADD(day, n-1, @startdt) AS dt
FROM dbo.GetNums(DATEDIFF(day, @startdt, @enddt) + 1) AS Nums;  
GO

use AdventureWorks2008R2
GO

DECLARE 
 @startdt AS DATE = '20050701',
 @enddt   AS DATE = '20080731'
 
SELECT DATEADD(month, n-1, @startdt) AS dt, SalesPersonID AS sp INTO #allMonths
FROM tempdb.dbo.GetNums(DATEDIFF(month, @startdt, @enddt) + 1) AS Nums
CROSS JOIN 
(SELECT DISTINCT SalesPersonID FROM Sales.SalesOrderHeader) AS A;  
GO

select * from #allmonths
where sp = 274

WITH SalesByMonth AS
(
SELECT sp as SalesPersonID,
       DATEPART(yyyy,dt) as Year,
	   DATEPART(MONTH,dt) as Month,
	   ISNULL(SalesTotal, 0) as SalesTotal
FROM #allMonths m
OUTER APPLY (
SELECT  SUM(SubTotal) AS SalesTotal
FROM    Sales.SalesOrderHeader 
WHERE   (SalesPersonID = m.sp OR (SalesPersonID IS NULL AND  m.sp IS NULL))
 AND    DATEPART(yyyy, OrderDate) = DATEPART(yyyy,dt) 
 AND    DATEPART(MONTH, OrderDate) = DATEPART(MONTH,dt)
) as t
)
select * into Sales.SalesByPersonByMonthAll from SalesByMonth
SELECT  SalesPersonID ,
        [Year] ,
        [Month] ,
        SalesTotal,
        LAG(SalesTotal, 1, 0.00) OVER (PARTITION BY SalesPersonID
                                  ORDER BY [Year], [Month]) AS last_month ,
        LAG(SalesTotal, 3, 0.00) OVER (PARTITION BY SalesPersonID
                                           ORDER BY [Year], [Month]) AS three_months_ago ,
        LAG(SalesTotal, 6, 0.00) OVER (PARTITION BY SalesPersonID
                                           ORDER BY [Year], [Month]) AS six_months_ago ,
        LAG(SalesTotal, 9, 0.00) OVER (PARTITION BY SalesPersonID
                                           ORDER BY [Year], [Month]) AS nine_months_ago ,
        LAG(SalesTotal, 12, 0.00) OVER (PARTITION BY SalesPersonID
                                            ORDER BY [Year], [Month]) AS twelve_months_ago 
FROM SalesByMonth
WHERE    SalesPersonID IS NOT NULL
ORDER BY SalesPersonID,
        [Year],
        [Month];

-- Note that LAG and LEAD are just shortcuts
SELECT  SalesPersonID ,
        [Year] ,
        [Month] ,
        SalesTotal,
        LAG(SalesTotal, 1) OVER (PARTITION BY SalesPersonID
                                  ORDER BY [Year], [Month]) AS last_month
FROM Sales.SalesByPersonByMonthAll
WHERE    SalesPersonID IS NOT NULL
ORDER BY SalesPersonID,
        [Year],
        [Month];

SELECT  SalesPersonID ,
        [Year] ,
        [Month] ,
        SalesTotal,
        MIN(SalesTotal) OVER (PARTITION BY SalesPersonID
                              ORDER BY [Year], [Month] ROWS BETWEEN 1 PRECEDING AND 1 PRECEDING) AS last_month
FROM Sales.SalesByPersonByMonthAll
WHERE    SalesPersonID IS NOT NULL
ORDER BY SalesPersonID,
        [Year],
        [Month];

--drop table Sales.SalesByMonthAll
